@@ -29,10 +29,12 @@ class Binary_CrossEntropy(Loss):
         super().__init__()
         
     def loss(self, targets, outputs):
-        return -(np.sum(targets * np.log(outputs + 1e-10) + (1-targets) * np.log(1-outputs + 1e-10))) / len(targets)
+        e = 1e-10
+        return -(np.sum(targets * np.log(outputs + e) + (1-targets) * np.log(1-outputs + e))) / len(targets)
     
     def derivative(self, targets, outputs):
-        return -(targets / outputs - (1 - targets + 1e-10) / (1 - outputs + 1e-10)) / len(outputs) 
+        e = 1e-10
+        return -(targets / outputs - (1 - targets + e) / (1 - outputs + e)) / len(outputs) 
     
 class Categorical_CrossEntropy(Loss):
     def __init__(self):
@@ -331,15 +333,15 @@ class Sequential():
         for i in range(len(self.train_layers)):
             if i+1 == len(self.train_layers):
                 self.train_outputs = self.train_layers[i].forward()
-                break
-            self.train_layers[i+1].inputs = self.train_layers[i].forward()
+            else:
+                self.train_layers[i+1].inputs = self.train_layers[i].forward()
         if self.val_inputs is not None and self.val_targets is not None:
             self.val_layers[0].inputs = self.val_inputs
             for i in range(len(self.val_layers)):
                 if i+1 == len(self.val_layers):
                     self.val_outputs = self.val_layers[i].forward()
-                    break
-                self.val_layers[i+1].inputs = self.val_layers[i].forward()
+                else:
+                    self.val_layers[i+1].inputs = self.val_layers[i].forward()
                 
     def predict(self,X):
             self.train_layers[0].inputs = X
@@ -347,15 +349,13 @@ class Sequential():
                 if i+1 == len(self.train_layers):
                     self.prediction = np.around(self.train_layers[i].forward())
                     return self.train_layers[i].forward()
-                    break
-                self.train_layers[i+1].inputs = self.train_layers[i].forward()
+                else:
+                    self.train_layers[i+1].inputs = self.train_layers[i].forward()
             
     def backward(self, lr, momentum):
         self.dX = self.train_cost_fn.derivative(self.train_targets, self.train_outputs)
-        self.train_layers.reverse()
-        for layer in self.train_layers:
+        for layer in reversed(self.train_layers):
                 self.dX = layer.backward(self.dX, lr, momentum)
-        self.train_layers.reverse()
         if self.val_inputs is not None and self.val_targets is not None:
             for i in range(len(self.val_layers)):
                 if isinstance(self.val_layers[i], Dense):
@@ -408,7 +408,7 @@ class Sequential():
         elif verbose == 0:
             return 
         else:
-            raise ValueError("unknown value for verbose")
+            raise ValueError("invalid value for 'verbose'")
     
     def fit(self, cost_fn, epochs = 1000, lr = 0.1, momentum = None, verbose = 1, live_stats = None):
         if live_stats == None:
@@ -426,16 +426,16 @@ class Sequential():
             if live_stats.type == n:
                 break
             elif n == 2 and live_stats.type != n:
-                raise ValueError("unknown value for live_stats.type")
+                raise ValueError("invalid value for 'live_stats.type'")
 
         if live_stats.row_select != "full" and live_stats.row_select != "limited":
-            raise ValueError("unknown value for live_stats.row_select")
+            raise ValueError("invalid value for 'live_stats.row_select'")
         
         if type(live_stats.f1) != int:
-            raise ValueError("unknown value for live_stats.f1")
+            raise ValueError("invalid value for 'live_stats.f1'")
         
         if type(live_stats.f2) != int:
-            raise ValueError("unknown value for live_stats.f2")
+            raise ValueError("invalid value for 'live_stats.f2'")
             
         self.__is_trained__ = True
         import copy
