@@ -192,4 +192,46 @@ class SimpleImputer():
                 else:
                     if self.X[i,self.column] == self.missing_values:
                         self.X[i,self.column] = float(mean)
-            return self.X
\ No newline at end of file
+            return self.X
+
+class BFFImputer():
+    def evaluate(self, pos):
+        import numpy as np
+        self.diffs = np.zeros(shape=self.X.shape)
+        self.scores = np.zeros(shape=(self.X.shape[0], 1))
+        for i in range(len(self.X[0,:])):
+            self.diffs[pos[0],i] = self.X[pos[0], i]
+            
+        for i in range(len(self.X[:,0])):
+            for j in range(len(self.X[0,:])):
+                if j != pos[1]: 
+                    self.diffs[i,j] = np.absolute(self.X[pos[0], j] - self.X[i,j])
+                    
+        for i in range(self.scores.shape[0]):
+            self.scores[i] = np.sum(self.diffs[i])
+        
+        return [np.argsort(self.scores, axis=0)[i] for i in range(self.scores.shape[0]//2) if np.isnan(self.X[np.argsort(self.scores, axis=0)[i][0], self.col]) == False]
+        
+    def transform(self, X, column):
+        import numpy as np
+        self.X = np.copy(X)
+        self.col = column
+        self.nans = []
+        self.diffs = np.zeros(shape=self.X.shape)
+        self.scores = np.zeros(shape=(self.X.shape[0], 1))
+        self.friends = []
+        
+        for i in range(len(self.X[0,:])):
+            if type(self.X[0,i]) == str:
+                for j in range(len(self.X[:,i])):
+                    self.X[j,i] = 0
+        
+        for i in range(len(self.X[:,self.col])):
+            if np.isnan(self.X[i,self.col]):
+                self.nans.append((i,self.col))
+        
+        for i in range(len(self.nans)):
+            self.friends = self.evaluate(self.nans[i])
+            self.X[self.nans[i][0], self.nans[i][1]] = self.X[self.friends[0][0], self.col]
+           
+        return self.X
\ No newline at end of file
